"use strict"

// @class EventDispatcher
//
// Simple event dispatcher for managing events in Mapache
class EventDispatcher {
    // Constructor for the event dispatcher, initialices the listeners
    constructor() {
        this.listeners = {}
    }

    // Adds a new Event Listener with the callback and the scope where the
    // callback is defined
    // @type - the name of the dispatched event
    // @callback - function
    // @scope - the scope where the @callback is defined
    addEventListener(type, callback, scope) {
        if(typeof this.listeners[type] !== "undefined") {
            this.listeners[type].push({scope:scope, callback:callback});
        } else {
            this.listeners[type] = [{scope:scope, callback:callback}];
        }
    }

    // Remove an Event Listener if its present
    // @type - the name of the dispatched event
    // @callback - function
    // @scope - the scope where the @callback is defined
    removeEventListener(type, callback, scope) {
        if(typeof this.listeners[type] !== "undefined") {
            let oldNumCallbacks = this.listeners[type].length;
            let newCallbacks = [];
            for(let i = 0; i < oldNumCallbacks; i++) {
                let oldListener = this.listeners[type][i];
                if(oldListener.scope != scope && oldListener.callback != callback) {
                    newCallbacks.push(oldListener);
                }
            }
            this.listeners[type] = newCallbacks;
        }
    }

    // Checks if the callback for the scope exists
    // @type - the name of the dispatched event
    // @callback - function
    // @scope - the scope where the @callback is defined
    hasEventListener(type, callback, scope) {
        if(typeof this.listeners[type] !== "undefined") {
            let numOfCallbacks = this.listeners[type].length;
            if(callback === undefined && scope === undefined) {
                return numOfCallbacks > 0
            }
            for(let i = 0; i < oldNumCallbacks; i++) {
                let listener = this.listeners[type][i];
                if((scope ? listener.scope == scope : true) && listener.callback == callback) {
                    return true;
                }
            }
        }
        return false;
    }

    // Dispatch the event to the listeners
    // @type - the name of the dispatched event
    // @target - the dispatcher
    // @arg - arguments of the event
    dispatch(type, target, ...args) {
        let event = {type:type, target:target};
        let callbackArgs = [{ type:type, target:target }].concat(args)
        if(typeof this.listeners[type] != "undefined") {
            let numOfCallbacks = this.listeners[type].length;
            for(var i = 0; i < numOfCallbacks; i++) {
                let listener = this.listeners[type][i];
                if(listener && listener.callback) {
                    listener.callback.apply(listener.scope, callbackArgs);
                }
            }
        }
    }
}

let eventDispatcher = new EventDispatcher();

module.exports = eventDispatcher;
