"use strict"

var CRSUtil = require('../utils/mp-crs-util.js');
var EventDispatcher = require('../event-dispatcher.js');

let instance = null;

// @class MpMap
//
// Stores the map and relevant information of it. Includes methods for managing
// and creating the layers of the map. Its the conection point with the
// Leaflet framework.
class MpMap {
    constructor() {
        if(!instance) {
            instance = this;
        }

        return instance;
    }

    // Renders the node map, stores the id and the MapComponent of the rendered
    // map, and move the center of the map to the position specified.
    renderMap(id, position, mapConfig) {
        this._mapConfig = mapConfig;
        this._createMap(position);
        this._id = id;
    }

    // Add a marker or layer to the current map. Once added to the DOM element,
    // append the content of the layer to it.
    //
    // @layer - Leaflet layer to be added to the map.
    // @element - DOM element with the content of the given layer or marker.
    addLayer(layer, element) {
        if(this._map !== undefined) {
            layer.marker.on('add', e => {
                layer.marker._icon.appendChild(element);
            });
            this._layers[layer.floor].nodes.addLayer(layer.marker);
        }
    }

    // Pans the map position to the given position in the arguments.
    //
    // @position - The new center of the map viewport.
    panTo(position) {
        this._map.panTo(position, {
            animate: true,
            duration: 0.5,
            easeLinearity: 1
        });
    }

    // Move the position of the marker.
    //
    // @layer - The marker to be moved.
    // @position - The new position of the marker.
    moveMarker(layer, position) {
        if(this._map !== undefined) {
            layer.marker.setLatLng(position);
        }
    }

    // Change the marker from its current floor inside an indoor map to
    // another floor.
    //
    // @layer - The layer of the marker.
    // @element - The marker to move.
    // @floor - The target floor.
    changeFloor(layer, element, floor) {
        if(this._map !== undefined) {
            this._layers[layer.floor].nodes.removeLayer(layer.marker);
            layer.floor = floor;
            this._layers[layer.floor].nodes.addLayer(layer.marker);
        }
    }

    // Remove a marker or a layer from the current map.
    //
    // @layer - The layer to be removed.
    removeLayer(layer) {
        if(this._map !== undefined && layer !== undefined) {
            this._map.removeLayer(layer.marker);
        }
    }

    // Create and return a new Leaflet marker with the position based on the
    // CRS of the map, and the class name for the marker.
    //
    // @position - Position for the marker.
    // @floor - The floor of the marker in the current map.
    // @crs - Reference System of the map.
    // @className - Class name for the DOM element.
    createMarker(position, floor, crs, className) {
        return {
            marker: L.marker(
                CRSUtil.getPosition(this, position),
                { icon: L.divIcon({ className: className }) }),
            floor: floor
        };
    }

    // Stores the reference of the current map and fires an event to restart
    // the server calls to retrieve the node data from the new map.
    //
    // @id - Id of the new map.
    changeMap(id) {
        if(id !== undefined) {
            this._oldId = this._id;
            this._id = id;
        }
        else {
            this._id = this._oldId;
        }

        EventDispatcher.dispatch('update', this, id);
    }

    // Create the Leaflet Map Layer with the map configuration.
    //
    // @position - The center of the viewport in the created map.
    _createMap(position) {
        if(this._map !== undefined) {
            this._clearMap();
        }

        this._map = L.map(this._mapHook, {
            crs: this._mapConfig._crs === 'outdoor'
                ? L.CRS.EPSG3857
                : L.CRS.Simple,
            minZoom: this._mapConfig._minZoom,
            maxZoom: this._mapConfig._maxZoom,
            zoomControl: false
        });

        this._setupFloors();
        this._setMapView(position);
        this._setMapControls();

        let layer = this._layers
            .find(l => l.main === true).layer;
        this._map.addLayer(layer);
    }

    // Remove all the layers and elements from the DOM of the current map.
    //
    // @map - The map layer
    _clearMap(map) {
        this._map.remove();
        while (this._mapHook.hasChildNodes()) {
            this._mapHook.removeChild(this._mapHook.lastChild);
        }
    }

    // Create the Leaflet Layers for each of the floors of the map.
    _setupFloors() {
        this._layers = [];
        this._mapConfig._floors.forEach(f => {
            let floor = {
                main: f.main,
                name: f.name,
                layer: CRSUtil.createMapLayer(this, f.tileURL),
                nodes: L.layerGroup()
            };

            floor.layer.addLayer(floor.nodes);
            this._layers.push(floor);
        });
    }

    // Set the viewport center of the map depending on the Current Reference
    // System of the map.
    //
    // @position - The center of the new map.
    _setMapView(position) {
        if(this._mapConfig._crs === 'outdoor') {
            this._map.setView(position, 16);
        }
        else {
            this._map.setView(this._map.unproject(
                [this._mapConfig._width/2, this._mapConfig._height/2],
                this._map.getMaxZoom()
            ), 1);

            const maxSouthWest = this._map.unproject(
                [(-1) * this._mapConfig._width, (3/2) * this._mapConfig._height],
                this._map.getMaxZoom()
            );
            const maxNorthEast = this._map.unproject(
                [2 * this._mapConfig._width, (-1) * this._mapConfig._height/2],
                this._map.getMaxZoom()
            );
            const maxBounds = L.latLngBounds(maxSouthWest, maxNorthEast);
            this._map.setMaxBounds(maxBounds);
        }
    }

    // Creates the control buttons for the map. Position the zoom control on
    // the lower right corner, and create a floor control panel selector if
    // the map has more than one floor.
    _setMapControls() {
        this._controlZoom = L.control.zoom({
            position: 'bottomright'
        });

        this._controlZoom.addTo(this._map);
        if(this._layers.length > 1) {
            let layersControl = {};
            this._layers.forEach(l => {
                layersControl[l.name] = l.layer;
            });

            this._controlFloor = L.control.layers(layersControl);
            this._controlFloor.addTo(this._map);
        }
    }

    set mapHook(hook) {
        this._mapHook = hook;
    }

    set id(id) {
        this._id = id;
    }

    get id() {
        return this._id;
    }

    get mapConfig() {
        return this._mapConfig;
    }
}

let map = new MpMap();

module.exports = map;
