"use strict"

var map = require('./map.js');

var ArrayUtil = require('../utils/mp-array-util.js');

var EventDispatcher = require('../event-dispatcher.js');

var Node = require('./mp-node.js');

var MainMenuView = require('../views/mp-main-menu.js');

// @class Mapache
//
// Stores al the nodes with their state, also
class Mapache {

    constructor() {
        this._changeMap = true;
        this._nodes = [];
        this._missingNodes = [];

        this._setupEventListeners();
    }

    // Starts the framework, getting the element hooks for the DOM and
    // starting the AJAX calls to the server.
    //
    // @hooks - The DOM elements used by Mapache in order to build all the
    // views.
    initMapache(hooks) {
        this._hooks = {
            map: document.getElementById(hooks.mapHook),
            menu: document.getElementById(hooks.menuHook)
        }
        map.mapHook = this._hooks.map;

        this.addWheelListener(this._hooks.map, this._addWheelEventListener);
        this._startServerCalls();
    }

    // Add an entry for the menu hook in the DOM.
    //
    // @view - the element to append.
    addMenu(view) {
        view.addTo(this._hooks.menu);
    }

    // Remove a node from the updated node list.
    //
    // @node - The node to be removed.
    removeNode(node) {
        ArrayUtil.removeFrom(this._nodes, 'id', node.id);
    }

    // Fired when the user scrolls the mouse wheel, dispatching an event
    // internally so the components could listen to it.
    //
    // @e - Mouse input event.
    _addWheelEventListener(e) {
        EventDispatcher.dispatch(`mousewheel`, this, e);
    }

    // AJAX call to retrieve the updated node from the REST API of the server.
    //
    // @id - Next map id to be rendered.
    _getNodeData(id) {
        let request = 'mapache?request=nodes';

        if(id !== undefined) {
            request += '&map=' + id;
        }

        if (this.xhr !== undefined) {
            this.xhr.abort();
        }

        return new Promise((resolve, reject) => {
            let xhr = new XMLHttpRequest();

            this.xhr = xhr;

            xhr.open("GET", request);
            xhr.onload = () => {
                if (xhr.status >= 200 && xhr.status < 300) {
                    resolve(xhr.response);
                } else {
                    reject(xhr.statusText);
                }
            };
            xhr.onerror = () => reject(xhr.statusText);
            xhr.send();
        });
    }

    // Restarts the automatic calls to the servers with the need map id
    // passed as argument.
    //
    // @event - EventDispatcher the event data from the fired event.
    // @id - Next map id to be rendered.
    _startServerCalls(event, id) {
        clearInterval(this._updateRate);

        this._changeMap = true;

        this._updateNodes(id);
        this._updateRate = setInterval(() => {
            this._updateNodes(id);
        }, 1000);
    }

    // Update the node list with the JSON provided by the server.
    //
    // @id - Next map id to be rendered.
    _updateNodes(id) {
        this._getNodeData(id).then(data => {
            if(data === '') {
                location.replace('/');
            }

            data = JSON.parse(data);

            if(this._changeMap) {
                if(this._nodes.length > 0) {
                    this._menuView.remove();
                    this._nodes.forEach(n => {
                        n.remove();
                    });
                    this._nodes = [];

                    this._missingNodes.forEach(n => {
                        n.remove();
                    });
                    this._missingNodes = [];
                }

                map.id = data.map.id;
                this._nodes.push(new Node(this, data.map));
                this._menuView = new MainMenuView(this._hooks.menu);

                data.nodes.forEach(d => {
                    try {
                        this._nodes.push(new Node(this, d));
                    } catch (e) {
                        console.error(e);
                    }
                });
                try {
                    this._nodes.forEach(n => { n.render(); });
                } catch (e) {
                    console.error(e);
                }
                this._changeMap = !this._changeMap;
            }
            else {
                this._missingNodes = ArrayUtil.outerIntersection(
                    this._nodes,
                    data.nodes,
                    (a, b) => a.id === b.id);

                data.nodes.forEach(d => {
                    try {
                        let node = this._nodes.find(n => n.id === d.id);
                        if(node === undefined) {
                            let node = new Node(this, d);
                            this._nodes.push(node);
                            node.render();
                        }
                        else {
                            node.update(d);
                        }
                    } catch (e) {
                        console.error(e);
                    }
                });
            }
        });
    }

    // Listens for the update event in order to restart the server calls.
    _setupEventListeners() {
        EventDispatcher.addEventListener(
            `update`,
            this._startServerCalls,
            this);
    }

    get nodes() {
        return this._nodes;
    }

    get missingNodes() {
        return this._missingNodes;
    }

    get menuView() {
        return this._menuView;
    }
}

module.exports = Mapache;
