"use strict"

var map = require('./map.js');

var ArrayUtil = require('../utils/mp-array-util.js');
var DOMUtil = require('../utils/mp-dom-util.js');
var EventDispatcher = require('../event-dispatcher.js');

var Decorator = require('./mp-decorator.js');

// @class ClusterDecorator
//
// Represents the node as a cluster of one or more nodes. When the defined
// zoom is reached, it tells the node to render their children.
class ClusterDecorator extends Decorator {
    constructor(component, node, data) {
        super(component, node);

        this._maxZoom = data.cluster.maxZoom;
        this._children = data.cluster.children;
    }

    // Modify the Marker View visibility based on the current map zoom level,
    // and append the children Item Views to this node Menu Item View.
    //
    // @marker - The Marker View of this node.
    // @menu - The Menu Item View of this node.
    render(marker, menu) {
        this._component.render(marker, menu);

        this._node.children = ArrayUtil.intersection(
            this._node.mapache.nodes,
            this._children,
            (a, b) => a.id === b);

        this._renderMarker(marker);
        this._renderMenu(menu);

        this._setupEventListeners();
    }

    // Update the cluster Menu Item View, appending or removing the
    // children specified by the server.
    update(marker, menu, data) {
        this._component.update(marker, menu, data);

        if(data !== undefined) {
            this._updateCluster(menu, data);
        }
    }

    // Remove the event listeners of this component.
    remove() {
        this._component.remove();

        this._removeEventListeners();
    }

    // Return true if the component searched is the cluster one.
    hasComp(comp) {
        if(comp === 'cluster') return true;
        return this._component.hasComp(comp);
    }

    // Set the visibility of this cluster node based on the current map zoom
    // level.
    //
    // @marker - The Marker View of this node.
    _renderMarker(marker) {
        let visibility = map._map.getZoom() < this._maxZoom;

        this._node.setVisibilityClusterState(this._node.id, visibility);
        this._node.children.forEach(c => {
            c.setVisibilityClusterState(this._node.id, !visibility);
        });
    }

    // Append the node Item View to the main app menu if the zoom level is
    // lower than one. Otherwise append the node children Item View to
    // this node Item View.
    //
    // @menu - The Menu Item View of this node.
    _renderMenu(menu) {
        if(this._maxZoom === -1) {
            this._node.mapache.addMenu(menu);
        }
        else {
            this._appendChildren(menu, this._node.children);
        }
    }

    // Remove or Append the new nodes specified by the server from the cluster
    // Item View.
    _updateCluster(menu, data) {
        this._children = data.cluster.children;

        let toAppend = ArrayUtil.outerIntersection(
            this._children,
            this._node.children,
            (a, b) => a === b.id);

        let toRemove = ArrayUtil.outerIntersection(
            this._node.children,
            this._children,
            (a, b) => a.id === b);

        let nodes = ArrayUtil.intersection(
            this._node.mapache.nodes,
            toAppend,
            (a, b) => a.id === b);

        this._node.children.push(nodes);

        this._node.children = ArrayUtil.flatten(this._node.children);

        if(nodes.length > 0) {
            this._appendChildren(menu, nodes);
        }
        if(toRemove.length > 0) {
            this._removeChildren(menu, toRemove);
        }

        if(this._maxZoom < 0) {
            this._node.children.forEach(c => {
                this._node.mapache.menuView.addMenuItem(this._node.name, c.menuItemView);
            });
        }
    }

    // Append the children inside the Menu Item View DOM element of this
    // node.
    //
    // @menu - The Menu Item View of this node.
    // @nodes - The nodes to append into @menu.
    _appendChildren(menu, nodes) {
        menu.appendChildren(nodes.map(e => e.menuItemView));
    }

    // Remove the children inside the Menu Item View DOM element of this
    // node.
    //
    // @menu - The Menu Item View of this node.
    // @nodes - The nodes to remove from @menu.
    _removeChildren(menu, nodes) {
        menu.removeChildren(nodes.map(e => e.menuItemView));
        nodes.forEach(n => {
            if (!this._node.mapache.missingNodes.some(n1 => n.id === n1.id)) {
                ArrayUtil.removeFrom(this._node.children, 'id', n.id);
                n.removeVisibilityClusterState(this._node.id);
            }
        });
    }

    // Fired when the user scrolls the mouse. Checks if the zoom reached the
    // maxZoom and renders the children into the map, or removes them.
    _zoomChanged(event, e) {
        if(e.deltaY < 0 && map._map.getZoom() + 1 >= this._maxZoom) {
            this._node.setVisibilityClusterState(this._node.id, false);
            this._node.children.forEach(c => {
                c.setVisibilityClusterState(this._node.id, true);
            });
        }
        else if(e.deltaY > 0 && map._map.getZoom() - 1 < this._maxZoom) {
            this._node.setVisibilityClusterState(this._node.id, true);
            this._node.children.forEach(c => {
                c.setVisibilityClusterState(this._node.id, false);
            });
        }
    }

    // Fired when the user selects this cluster in the main menu to show
    // the current cluster menu.
    _showMenu(event) {
        this._node.mapache.menuView.showMenu(this._node.name);

        this._node.children.forEach(c => {
            this._node.mapache.menuView.addMenuItem(this._node.name, c.menuItemView);
        });
    }

    _setupEventListeners() {
        if(this._maxZoom === -1) {
            EventDispatcher.addEventListener(
                `node-${this._node.id}-selected`,
                this._showMenu,
                this);
        }

        EventDispatcher.addEventListener(
            `mousewheel`,
            this._zoomChanged,
            this);
    }

    _removeEventListeners() {
        if(this._maxZoom === -1) {
            EventDispatcher.removeEventListener(
                `node-${this._node.id}-selected`,
                this._showMenu,
                this);
        }

        EventDispatcher.removeEventListener(
            `mousewheel`,
            this._zoomChanged,
            this);
    }
}

module.exports = ClusterDecorator;
