"use strict"

var DOMUtil = require('../utils/mp-dom-util.js');
var ArrayUtil = require('../utils/mp-array-util.js');
var EventDispatcher = require('../event-dispatcher.js');

var Decorator = require('./mp-decorator.js');

var MenuDataView = require('../views/mp-menu-data.js');

// @class DataDecorator
//
// Decorates the node with a data menu to show information other than
// position, mostly related with Sensorization data.
class DataDecorator extends Decorator {
    constructor(component, node, data) {
        super(component, node);

        this._data = data.data;
    }

    // Create the Menu for the data of this component.
    //
    // @marker - The node Marker View.
    // @menu - The node Menu View.
    render(marker, menu) {
        this._component.render(marker, menu);

        this._renderMenu(menu);
        this._setupEventListeners();
    }

    // Update the data of the Menu Data of this node.
    //
    // @marker - The node Marker View.
    // @menu - The node Menu View.
    update(marker, menu, data) {
        this._component.update(marker, menu, data);

        if(data !== undefined) {
            if(data.data !== this._data) {
                this._updateMenu(data.data);
            }
        }
    }

    // Remove the listeners of this component.
    remove() {
        this._component.remove();

        this._removeEventListeners();
    }

    // Return true if the component searched is the data one.
    hasComp(comp) {
        if(comp === 'data') return true;
        return this._component.hasComp(comp);
    }

    // Create the Menu Data View and the content for this menu.
    _renderMenu() {
        this._menu = new MenuDataView(this._node.id);

        this._createMenuContent(this._data);
    }

    // Update the Menu Data View, removing data no longer present on the node,
    // and appending the new one. Also, update the data that currently exists.
    _updateMenu(data) {
        let toAppend = ArrayUtil.outerIntersection(
            data,
            this._data,
            (a, b) => a.name === b.name);

        let toRemove = ArrayUtil.outerIntersection(
            this._data,
            data,
            (a, b) => a.name === b.name);

        if(toAppend.length > 0) {
            this._createMenuContent(toAppend);
        }

        if(toRemove.length > 0) {
            toRemove.forEach(d => {
                this._menu.removeOverviewItem(d.name);
                this._menu.removeRelativeOverviewItem(d.name);
                this._menu.removeHistoryItem(d.name);
            });
        }

        this._data.forEach(d => {
            this._menu.updateOverviewItem({
                name: d.name,
                value: d.values[d.values.length - 1].value,
                unit: d.unit
            })
        });

        let relativeData = this._data.filter(d => d.max !== undefined);
        relativeData.forEach(d => {
            this._menu.updateRelativeOverviewItem({
                name: d.name,
                value: d.values[d.values.length - 1].value,
                unit: d.unit,
                max: d.max
            })
        });

        let historyData = this._data.find(d => d.name === this._selectedData);
        if (historyData !== undefined) {
            this._menu.updateHistoryItem(historyData);
        }

        this._data = data;
    }

    // Create the Elements in the Data Menu View for each type of data received
    // from the server.
    //
    // @data - The concrete data array to be rendered in the Menu Data View.
    _createMenuContent(data) {
        data.forEach(d => {
            this._menu.setOverviewItem({
                name: d.name,
                value: d.values[d.values.length - 1].value,
                unit: d.unit
            })
        });

        let relativeData = data.filter(d => d.max !== undefined);
        relativeData.forEach(d => {
            this._menu.setRelativeOverviewItem({
                name: d.name,
                value: d.values[d.values.length - 1].value,
                unit: d.unit,
                max: d.max
            })
        });

        let historyData = data.filter(d => d.values.length > 1);
        historyData.forEach(d => {
            this._menu.setHistoryItem(d);
        });

        if(historyData.length > 0) {
            this._selectedData = historyData[0].name;
        }
    }

    // Fired when the user selects this node, it displays the Menu Data View
    // associated with this node.
    //
    // @event - The data of the fired Event from the EventDispatcher.
    _showMenu(event) {
        this._node.mapache.menuView.showMenu(this._node.name);
        this._node.mapache.menuView.addMenuItem(this._node.name, this._menu);
    }

    // Fired when the user selects a concrete data from all the concrete data
    // of this node. It changes the data displayed in the Menu Data View
    // chart.
    //
    // @event - The data of the fired Event from the EventDispatcher.
    // @name - Name of the selected data.
    _dataSelected(event, name) {
        if(this._selectedData !== name) {
            this._selectedData = name;
            this._menu.clearHistoryItem();
        }
    }

    _setupEventListeners() {
        EventDispatcher.addEventListener(
            `data-${this._node.id}-selected`,
            this._dataSelected,
            this);

        EventDispatcher.addEventListener(
            `node-${this._node.id}-selected`,
            this._showMenu,
            this);
    }

    _removeEventListeners() {
        EventDispatcher.removeEventListener(
            `data-${this._node.id}-selected`,
            this._dataSelected,
            this);

        EventDispatcher.removeEventListener(
            `node-${this._node.id}-selected`,
            this._showMenu,
            this);
    }
}

module.exports = DataDecorator;
