"use strict"

var map = require('./map.js');

var CRSUtil = require('../utils/mp-crs-util.js');
var EventDispatcher = require('../event-dispatcher.js');

var Decorator = require('./mp-decorator.js');

// @class MapDecorator
//
// Represents the node as a map. When clicked, the new map is rendered on top
// of the old one, and replaces the root tree node on Mapache.
class MapDecorator extends Decorator {
    constructor(component, node, data) {
        super(component, node);

        this._crs = data.map.crs;
        this._minZoom = data.map.minZoom || 0,
        this._maxZoom = data.map.maxZoom || (this._crs === 'outdoor' ? 18 : 6),
        this._height = data.map.height;
        this._width = data.map.width;
        this._floors = data.map.floors;
        this._layers = [];


        if(map.id === this._node.id || map.id === undefined) {
            let position = [data.position.x, data.position.y]
            map.renderMap(this._node.id, position, this);
        }

        this._setupEventListeners();
    }

    // There is nothing to render in the map as a View, but propagates the
    // call throught the nested components.
    render(marker, menu) {
        this._component.render(marker, menu);
    }

    // There is nothing to update in the map as a View, but propagates the
    // call throught the nested components.
    update(marker, menu, data) {
        this._component.update(marker, menu, data);
    }

    // Remove the listeners of the map component.
    remove() {
        this._component.remove();

        this._removeEventListeners();
    }

    // Return true if the component searched is the map one.
    hasComp(comp) {
        if(comp === 'map') return true;
        return this._component.hasComp(comp);
    }

    // Change the current map for this one.
    //
    // @event - The object that fired the event.
    _selectedEvent(event) {
        map.changeMap(this._node.id);
    }

    // When zoomed out, change the map to the last one rendered.
    _changeToOutterMap() {
        if(map._oldId !== this._node.id && map.mapConfig.crs !== 'outdoor') {
            map.changeMap();
        }
    }

    // When zoomed in, change the map to the map that the cursor is hovering
    // on.
    _changeToInnerMap() {
        let i = 0;
        let hoveredEls = document.querySelectorAll(':hover');
        let e = {};
        let selectedNodeId = {};

        for(i = 0; i < hoveredEls.length; i++) {
            e = hoveredEls.item(i);
            if((' ' + e.className + ' ').indexOf(' mp-node-marker-container ') > -1) {
                selectedNodeId = e.id.split('-')[3];
                if(selectedNodeId === this._node.id) {
                    map.changeMap(this._node.id);
                }
            }
        }
    }

    // Event fired every time the user scrolls to change the map zoom.
    //
    // @event - The object that fired the event.
    // @e - The object containing the input data of the scroll event.
    _zoomChanged(event, e) {
        if(e.deltaY > 0 && map._map.getZoom() === map._mapConfig._minZoom) {
            this._changeToOutterMap();
        }
        else if(e.deltaY < 0 && map._map.getZoom() === map._mapConfig._maxZoom) {
            this._changeToInnerMap();
        }
    }

    _setupEventListeners() {
        EventDispatcher.addEventListener(
            `mousewheel`,
            this._zoomChanged,
            this);
        EventDispatcher.addEventListener(
            `node-${this._node.id}-selected`,
            this._selectedEvent,
            this);
    }

    _removeEventListeners() {
        EventDispatcher.removeEventListener(
            `mousewheel`,
            this._zoomChanged,
            this);
        EventDispatcher.removeEventListener(
            `node-${this._node.id}-selected`,
            this._selectedEvent,
            this);
    }

    get crs() {
        return this._crs;
    }
}

module.exports = MapDecorator;
