"use strict"

var map = require('./map.js');

var EventDispatcher = require('../event-dispatcher.js');

var DOMUtil = require('../utils/mp-dom-util.js');
var ArrayUtil = require('../utils/mp-array-util.js');

var MapDecorator = require('./mp-map-decorator.js');
var DataDecorator = require('./mp-data-decorator.js');
var StateDecorator = require('./mp-state-decorator.js');
var PositionElement = require('./mp-position-element.js');
var ClusterDecorator = require('./mp-cluster-decorator.js');
var TooltipDecorator = require('./mp-tooltip-decorator.js');
var TimeoutDecorator = require('./mp-timeout-decorator.js');

var MapNodeView = require('../views/mp-map-node.js');
var MenuItemView = require('../views/mp-menu-item.js');

// @class Node
//
// Core element of the framework, a Node represent a single object of
// positioning in its most basic form. It has methods to add more
// functionality on the fly.
class Node {
    constructor(mapache, data) {
        this._id = data.id;
        this._name = data.name;
        this._mapache = mapache;
        this._visibilityState = true;
        this._clustersState = [];
        this._children = [];

        this._createComponents(data);
        this._setupEventListeners();
    }

    // Render the views of the node.
    render() {
        this._markerView = new MapNodeView(this._id);
        this._menuItemView = new MenuItemView(this._id);

        this._component.render(
            this._markerView,
            this._menuItemView
        );
    }

    // Update the node state with the new data provided by the server.
    update(data) {
        this._component.update (
            this._markerView,
            this._menuItemView,
            data
        );
    }

    // Remove the views and the listeners of the node and its components.
    remove() {
        this._removeNodeComponent();
        this._removeViews();
        this._removeEventListeners();
    }

    // Set the visibility selected by the user.
    //
    // @state - Whether the node should be visible or not.
    setVisibilityState(state) {
        this._visibilityState = state;

        if(this._markerView !== undefined) {
            this._markerView.visibility = this.visibility();
        }

        this._children.forEach(c => {
            c.setVisibilityState(state);
        });
    }

    // Set the visibility of the node based on the clusters it belongs.
    //
    // @id - Id of the parent cluster.
    // @state - Visibility state of the parent cluster.
    setVisibilityClusterState(id, state) {
        let clusterState = this._clustersState.find(c => c.id === id);

        if(clusterState === undefined) {
            this._clustersState.push({ id: id, state: state});
        }
        else {
            clusterState.state = state;
        }

        if(this._markerView !== undefined) {
            this._markerView.visibility = this.visibility();
        }
    }

    // Remove the visibility caused by a parent cluster node.
    //
    // @id - ID of the clsuter to be removed.
    removeVisibilityClusterState(id) {
        ArrayUtil.removeFrom(this._clustersState, 'id', id);

        if(this._markerView !== undefined) {
            this._markerView.visibility = this.visibility();
        }
    }

    // Returns the visibility of the marker based on its own visibility and
    // the visibility of its cluster parents.
    visibility() {
        let visibility = this._visibilityState;

        this._clustersState.forEach(c => {
            visibility = visibility && c.state;
        });

        return visibility;
    }

    // Create the components from the data received from the server. Each
    // component gives the node new functionality, so its recomended to
    // see the code or the documentation of each one.
    //
    // @data - Data send from the server of the current node.
    _createComponents(data) {
        let currentComp = new PositionElement(this, data);

        if(data.tooltip !== undefined) {
            currentComp = new TooltipDecorator(currentComp, this, data);
        }
        if(data.state !== undefined) {
            currentComp = new StateDecorator(currentComp, this, data);
        }
        if(data.timeout !== undefined) {
            currentComp = new TimeoutDecorator(currentComp, this, data);
        }
        if(data.cluster !== undefined) {
            currentComp = new ClusterDecorator(currentComp, this, data);
        }
        if(data.map !== undefined) {
            currentComp = new MapDecorator(currentComp, this, data);
        }
        if(data.data !== undefined) {
            currentComp = new DataDecorator(currentComp, this, data);
        }

        this._component = currentComp;
    }

    // Remove all the component data and listeners for each component of this
    // node.
    _removeNodeComponent() {
        this._component.remove();
        if(map.id !== this._id && this._elements !== undefined) {
            map.removeLayer(this._elements.layer);
        }
    }

    // Remove all the views of this node and the listeners binded to it.
    _removeViews() {
        if(this._menuItemView !== undefined) this._menuItemView.remove();
        if(this._markerView !== undefined) this._markerView.remove();
    }

    // Toggle the visibility of this node.
    _toggleVisibility() {
        this.setVisibilityState(!this._visibilityState);
    }

    _setupEventListeners() {
        EventDispatcher.addEventListener(
            `node-${this._id}-toggle-visibility`,
            this._toggleVisibility,
            this);
    }

    _removeEventListeners() {
        EventDispatcher.removeEventListener(
            `node-${this._id}-toggle-visibility`,
            this._toggleVisibility,
            this);
    }

    get id() {
        return this._id;
    }

    get name() {
        return this._name;
    }

    get mapache() {
        return this._mapache;
    }

    set children(children) {
        this._children = children
    }

    get children() {
        return this._children;
    }

    get component() {
        return this._component;
    }

    get menuItemView() {
        return this._menuItemView;
    }

    get marker() {
        return this._markerView;
    }
}

module.exports = Node;
