"use strict"

var map = require('./map.js');

var DOMUtil = require('../utils/mp-dom-util.js');
var CRSUtil = require('../utils/mp-crs-util.js');
var NodeComponent = require('./mp-node-component.js');
var EventDispatcher = require('../event-dispatcher.js');

// @class PositionElement
//
// Marker component used to create the leaflet marker with the given position.
class PositionElement extends NodeComponent {
    constructor(node, data) {
        super(node);

        let position = data.position;

        this._position = [position.x, position.y];
        this._z = position.z || 0;
        this._icon = data.icon || 'place';
        this._container = position.container;
        this._pannable = position.pannable;
    }

    // Create the DOM Elements for the menu and the map.
    //
    // @marker - Marker View of this node.
    // @menu - Menu View of this node.
    render(marker, menu) {
        if(map.id !== this._node.id || map.id === undefined) {
            this._renderMarker(marker);
            this._renderMenu(menu);
        }

        this._setupEventListeners();
    }

    // Update the position of the node if changed.
    //
    // @marker - Marker View of this node.
    // @menu - Menu View of this node.
    // @data - New data received from server.
    update(marker, menu, data) {
        if(data !== undefined) {
            this._updateMarker(marker, data);
        }
    }

    // Remove the listeners of this component.
    remove(removeListeners) {
        this._removeEventListeners();
    }

    // Return true if the component searched is the position one.
    hasComp(comp) {
        if(comp === 'position') return true;
        return false;
    }

    // Create the leaflet layer providing the position. If the position has a
    // container, the node DOM element renders inside the node container
    // Element.
    //
    // @marker - Marker View of this node.
    _renderMarker(marker) {
        marker.createLayer(this._position, this._z);

        if(this._container !== undefined) {
            this._parent = this._node.mapache.nodes
                .find(n => n.id === this._container);

            marker.removeLayer();
            this._parent.marker.appendChild(this._node.marker);
        }
        else {
            marker.addToMap();
        }

        marker.icon = this._icon;
    }

    // Change the menu item view icon and name.
    //
    // @menu - Menu Item View of this node.
    _renderMenu(menu) {
        menu.icon = this._icon;
        menu.name = this._node.name;
    }

    // Update the node marker position if the position changed.
    //
    // @marker - Marker View of the node.
    // @data - Updated data of this node.
    _updateMarker(marker, data) {
        if(this._position[0] !== data.position.x || this._position[1] !== data.position.y) {
            this._position = [data.position.x, data.position.y];
            marker.moveLayer(this._position);
        }
        if(this._z !== data.position.z) {
            this._z = data.position.z;
            marker.moveLayerFloor(this._z);
        }
        if (data.position.container === undefined && this._parent !== undefined) {
            this._container = undefined;

            marker.addToMap();

            this._parent = undefined;
        }
        else if(this._container !== data.position.container) {
            this._container = data.position.container;

            this._parent = this._node.mapache.nodes
                .find(n => n.id === this._container);

            marker.removeLayer();
            this._parent.marker.appendChild(this._node.marker);
        }
    }

    // When this node is clicked, pans the map to center the viewport in this
    // node position.
    _panToNode() {
        if(this._pannable) {
            map.panTo(this._position);
        }
    }

    _setupEventListeners() {
        EventDispatcher.addEventListener(
            `node-${this._node.id}-selected`,
            this._panToNode,
            this);
    }

    _removeEventListeners() {
        EventDispatcher.removeEventListener(
            `node-${this._node.id}-selected`,
            this._panToNode,
            this);
    }
}

module.exports = PositionElement;
