"use strict"

var ArrayUtil = require('../utils/mp-array-util.js');
var DOMUtil = require('../utils/mp-dom-util.js');
var EventDispatcher = require('../event-dispatcher.js');
var Decorator = require('./mp-decorator.js');

// @class StateDecorator
//
// Decorates the node to be able to show a state of itself.
class TimeoutDecorator extends Decorator {
    constructor(component, node, data) {
        super(component, node);

        this._deleteTime = data.timeout.deleteTime;
        this._warningTime = data.timeout.warningTime;
        this._state = data.timeout.warningState;
        this._initialState = data.state;
    }

    // Start the timeouts of this node, both remove and update.
    //
    // @marker - Marker View of this node.
    // @menu - Menu View of this node.
    render(marker, menu) {
        this._component.render(marker, menu);

        this._setTimeouts(marker, menu);
    }

    // Clear the timeouts if any data of this node is updated.
    //
    // @marker - Marker View of this node.
    // @menu - Menu View of this node.
    // @data - New data received from server.
    update(marker, menu, data) {
        this._component.update(marker, menu, data);

        if(data !== undefined) {
            this._clearTimeouts();

            this._deleteTime = data.timeout.deleteTime;
            this._warningTime = data.timeout.warningTime;
            this._state = data.timeout.warningState;
            this._initialState = data.state;

            marker.state = this._initialState;
            menu.state = this._initialState;

            this._setTimeouts(marker, menu);
        }
    }

    // Remove the timeouts of this node.
    remove() {
        this._clearTimeouts();

        this._component.remove();
    }

    // Return true if the component searched is the timeout one.
    hasComp(comp) {
        if(comp === 'timeout') return true;
        return this._component.hasComp(comp);
    }

    // Update the marker state if changed.
    //
    // @marker - Marker View of this node.
    _updateMarker(marker) {
        marker.state = this._state;
    }

    // Update the menu View state if changed.
    //
    // @menu - Menu View of this node.
    _updateMenu(menu) {
        menu.state = this._state;
    }

    // Set the timeouts for the given timers. If the warning timeout is
    // reached, it shows the new warning state. If the delete timeout is
    // reached, it removes the node from Mapache.
    //
    // @marker - Marker View of this node.
    // @menu - Menu View of this node.
    _setTimeouts(marker, menu) {
        this._warningTimer = setTimeout(() => {
            this._updateMarker(marker);
            this._updateMenu(menu);
        }, this._warningTime);

        this._deleteTimer = setTimeout(() => {
            this._node.remove();
            this._node.mapache.removeNode(this._node);
        }, this._deleteTime);
    }

    // Clear the node timeouts so they dont trigger the callback funtions if
    // the node has been updated.
    _clearTimeouts() {
        clearTimeout(this._warningTimer);
        clearTimeout(this._deleteTimer);
    }
}

module.exports = TimeoutDecorator;
