"use strict"

var map = require('../models/map.js');

var DOMUtil = require('../utils/mp-dom-util.js');
var ArrayUtil = require('../utils/mp-array-util.js');
var EventDispatcher = require('../event-dispatcher.js');

// @class MainMenuView.
//
// Create the DOM Element tree that represents and renders the main menu of
// the framework.
class MainMenuView {
    constructor(menu) {
        this._element = DOMUtil.createElement({
            type: 'div',
            id: `mp-menu`,
            class: 'mp-menu menu-hide'
        });
        this._menus = [];

        this.eventBack = this.back.bind(this);
        this.eventHide = this.hide.bind(this);

        this._addHeader();
        this._addButtons();

        menu.appendChild(this._element);
    }

    // Remove the current menu Element displayed and Renders the top level one.
    show() {
        let nextMenu = this._menus[this._menus.length - 1];
        let currentMenu = document.querySelectorAll('.mp-menu-container');

        [].forEach.call(currentMenu, m => {
            m.remove();
        });

        this._header.innerHTML = nextMenu.name;
        this._element.appendChild(nextMenu.element);

        DOMUtil.removeClass(this._element, 'menu-hide');
    }

    // Hide the Menu Element.
    hide() {
        if(this._menus.length > 0) {
            let currentMenu = this._menus[this._menus.length - 1];
            currentMenu.element.remove();
        }
        this._header.innerHTML = '';
        this._menus = [];
        DOMUtil.addClass(this._element, 'menu-hide');
    }

    // Remove the current Menu Element displayed and render the previous one
    // on the stack.
    back() {
        if(Array.isArray(this._menus)) {
            let currentMenu = this._menus.pop();
            let nextMenu = this._menus[this._menus.length - 1];

            currentMenu.element.remove();

            if(this._menus.length === 0) {
                this.hide();
            }
            else {
                this._header.innerHTML = nextMenu.name;
                this._element.appendChild(nextMenu.element);
            }
        }
    }

    // Add the view Element to the menu container of this view.
    //
    // @name - Name of the node view to show it in the header.
    // @view - Menu Item View to append this View.
    addMenuItem(name, view) {
        if(this.header === name) {
            let menu = this._menus.find(m => m.name === name);
            let children = menu.element.childNodes;
            let element = view.element;

            if (element.parentNode != menu.element) {
                menu.element.appendChild(element);
            }
        }
    }

    // Show the menu specified by the name given as parameter.
    //
    // @name - Name of the node view.
    showMenu(name) {
        this._header.innerHTML = name;
        this._addMenu(name);
        this.show();
    }

    // Remove all the emements from the DOM, and their listeners.
    remove() {
        this._element.remove();

        this._element.removeEventListener('click', this.eventBack);
        this._element.removeEventListener('click', this.eventHide);
    }

    // Create the header Element for this Menu. Later, this header will be
    // modified depending on the node thats being rendered on this menu.
    _addHeader() {
        let menuHeader = DOMUtil.createElement({
            type: 'div',
            class: 'mp-menu-header'
        });
        this._element.appendChild(menuHeader);

        this._header = DOMUtil.createElement({
            type: 'h3',
            class: 'mp-menu-header-title',
        });
        menuHeader.appendChild(this._header);
    }

    // Create the back and close buttons and add their event listeners for
    // the user input.
    _addButtons() {
        this._backButton = DOMUtil.createElement({
            type: 'i',
            class: 'material-icons mp-menu-back-btn',
            innerHTML: 'arrow_back'
        });
        this._backButton.addEventListener('click', this.eventBack);
        this._element.appendChild(this._backButton);

        this._closeButton = DOMUtil.createElement({
            type: 'i',
            class: 'material-icons mp-menu-close-btn',
            innerHTML: 'clear'
        });
        this._closeButton.addEventListener('click', this.eventHide);
        this._element.appendChild(this._closeButton);
    }

    // Create a new menu with the given name for this menu.
    _addMenu(name) {
        let menu = this._menus.find(m => m.name === name);
        if(menu !== undefined) {
            menu.element.remove();
            ArrayUtil.removeFrom(this._menus, 'name', name);
        }

        let content = DOMUtil.createElement({
            type: 'div',
            class: 'mp-menu-container'
        });

        this._menus.push({ name: name, element: content });
    }

    get element() {
        return this._element;
    }

    get header() {
        return this._header.innerHTML;
    }
}

module.exports = MainMenuView;
