"use strict"

var map = require('../models/map.js');

var DOMUtil = require('../utils/mp-dom-util.js');
var EventDispatcher = require('../event-dispatcher.js');

// @class MapNodeView.
//
// Create the DOM Element tree that represents and renders the marker on the
// map.
class MapNodeView {
    constructor(id) {
        this._id = id;

        this._element = DOMUtil.createElement({
            type: 'div',
            id: `mp-node-marker-${this._id}`,
            class: 'mp-node-marker-container'
        });

        this._addContainer();

        this.nodeSelected = this._nodeSelected.bind(this);

        this._setupEventListeners();
    }

    // Adds the current node marker Element to the
    addToMap() {
        map.addLayer(this._layer, this._element);
    }

    // Create the marker DOM Element as a Leaflet Layer with the position
    // of this node.
    createLayer(position, z) {
        this._layer = map.createMarker(position, z, 'mp-marker');
    }

    // Add a tooltip Element to the marker DOM Element container.
    addTooltip(tooltip) {
        this._tooltip = DOMUtil.createElement({
            type: 'div',
            class: 'mp-node-marker-tooltip',
            innerHTML: tooltip
        });

        this._element.appendChild(this._tooltip);
    }

    // Add a state to the marker. If state icon is true, create a SVGElement
    // and append it to the node Element. Otherwise change the color of the
    // marker Element.
    addState(state) {
        this._stateIcon = state.icon
            ? DOMUtil.createSVG({
                type: 'circle',
                height: 10,
                width: 10,
                color: state.color
            })
            : this._icon;

        if(state.icon) {
            this._element.appendChild(this._stateIcon);
            this._stateIcon = this._stateIcon.getElementsByTagName('circle')[0];
            this._stateIcon.style.fill = state.color;
        }
        else {
            this._stateIcon.style.color = state.color;
        }
    }

    // Move the marker to a new position.
    moveLayer(position) {
        map.moveMarker(this._layer, position);
    }

    // Move the marker from one floor to another.
    moveLayerFloor(floor) {
        map.changeFloor(this._layer, this._element, floor);
    }

    // Append to the children container a child of this marker.
    appendChild(view) {
        this._childList.appendChild(view.element);
    }

    // Remove the DOM Element of this view and all the listeners associated to
    // it.
    remove() {
        this._element.remove();
        this._element.removeEventListener('click', this.nodeSelected);
        map.removeLayer(this._layer);
    }

    // Remove this node marker from the map.
    removeLayer() {
        map.removeLayer(this._layer);
    }

    // Create the icon Element using material design icons, and the children
    // container Element.
    _addContainer() {
        this._icon = DOMUtil.createElement({
            type: 'i',
            class: 'material-icons',
            innerHTML: 'place'
        });
        this._element.appendChild(this._icon);

        this._childList = DOMUtil.createElement({
            type: 'div',
            class: 'mp-node-marker-children'
        });
        this._element.appendChild(this._childList);
    }

    // Fired when the DOM element of this node is selected.
    _nodeSelected(e) {
        e.stopPropagation();
        EventDispatcher.dispatch(`node-${this._id}-selected`, this);
    }

    _setupEventListeners() {
        this._element.addEventListener('click', this.nodeSelected);
    }

    get element() {
        return this._element;
    }

    set icon(icon) {
        this._icon.innerHTML = icon;
    }

    set visibility(state) {
        state
            ? DOMUtil.removeClass(this._element, 'node-hide')
            : DOMUtil.addClass(this._element, 'node-hide');
    }

    set state(state) {
        if(this._stateIcon.style.fill) {
            this._stateIcon.style.fill = state.color;
        }
        else {
            this._stateIcon.style.color = state.color;
        }
    }
}

module.exports = MapNodeView;
