"use strict"

var map = require('../models/map.js');

var DOMUtil = require('../utils/mp-dom-util.js');
var ArrayUtil = require('../utils/mp-array-util.js');
var EventDispatcher = require('../event-dispatcher.js');

// @class MenuDataView.
//
// Create the DOM Element tree that represents and renders the menu data item
// of a node. This Element will be rendered inside the Main Menu container
// Element.
class MenuDataView {
    constructor(id) {
        this._id = id;
        this._element = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-container'
        });

        this._overviewEls = [];
        this._relOverviewEls = [];
        this._charts = [];
    }

    // Remove this view Element from the DOM.
    remove() {
        this._element.remove();
    }

    // Create an overview Elememnt for the specified data and append it to the
    // overview data container. If the overview data container doesnt exists,
    // create one.
    //
    // @data - The data to create an overview Element.
    setOverviewItem(data) {
        if(this._overviewCard === undefined) {
            this._addOverview();
        }

        let overview = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview-item'
        });
        this._overviewCard.appendChild(overview);

        let name = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview-item-name',
            innerHTML: data.name
        });
        overview.appendChild(name);

        let value = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview-item-value',
            innerHTML: `${Number(data.value).toFixed(2)}<small>${data.unit}</small>`
        });
        overview.appendChild(value);

        this._overviewEls.push({
            element: overview,
            name: name,
            value: value,
        });
    }

    // Update the overview Element.
    //
    // @data - The data to update an overview Element.
    updateOverviewItem(data) {
        let element = this._overviewEls.find(e => e.name.innerHTML === data.name);

        if(element !== undefined) {
            element.value.innerHTML = `${Number(data.value).toFixed(2)}<small>${data.unit}</small>`;
        }
    }

    // Remove the overview Element.
    //
    // @name - The data name to remove Element.
    removeOverviewItem(name) {
        let item = this._overviewEls.find(e => e.name.innerHTML === name);
        if(item !== undefined) {
            ArrayUtil.removeFrom(this._overviewEls, 'name', item.name);
            item.element.remove();
        }
    }

    // Create the overview Element container.
    _addOverview() {
        let overview = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview'
        });
        this._element.appendChild(overview);

        let header = DOMUtil.createElement({
            type: 'h4',
            class: 'mp-data-overview-header',
            innerHTML: 'Datos Actuales'
        });
        overview.appendChild(header);

        this._overviewCard = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-card'
        });
        overview.appendChild(this._overviewCard);
    }

    // Create a relative overview Element for the specified data and append it
    // to the relative overview data container. If the relative overview data
    // container doesnt exists, create one.
    //
    // @data - The data to create a relative overview Element.
    setRelativeOverviewItem(data) {
        if(this._relOverviewCard === undefined) {
            this._addRelativeOverview();
        }

        let overview = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-rel-overview-item'
        });
        this._relOverviewCard.appendChild(overview);

        let value = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview-item-value',
            innerHTML: `${Number(data.value).toFixed(2)}<small>${data.unit}</small>`
        });
        overview.appendChild(value);

        let bar = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-bar'
        });
        overview.appendChild(bar);

        let barFill = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-bar-fill'
        });
        barFill.style.width = (data.value / data.max) * 100 + '%';
        bar.appendChild(barFill);

        let name = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview-item-name',
            innerHTML: data.name
        });
        overview.appendChild(name);

        this._relOverviewEls.push({
            element: overview,
            name: name,
            value: value,
            bar: barFill
        });
    }

    // Update the  relative overview Element.
    //
    // @data - The data to update a relative overview Element.
    updateRelativeOverviewItem(data) {
        let element = this._relOverviewEls.find(e => e.name.innerHTML === data.name);

        if(element !== undefined) {
            element.value.innerHTML = `${Number(data.value).toFixed(2)}<small>${data.unit}</small>`;
            element.bar.style.width = (data.value / data.max) * 100 + '%';
        }
    }

    // Remove the overview Element.
    //
    // @name - The data name to remove Element.
    removeRelativeOverviewItem(name) {
        let item = this._relOverviewEls.find(e => e.name.innerHTML === name);
        if(item !== undefined) {
            ArrayUtil.removeFrom(this._relOverviewEls, 'name', item.name);
            item.element.remove();
        }
    }

    // Create the relative overview Element container.
    _addRelativeOverview() {
        let overview = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-overview'
        });
        this._element.appendChild(overview);

        let header = DOMUtil.createElement({
            type: 'h4',
            class: 'mp-data-overview-header',
            innerHTML: 'Datos'
        });
        overview.appendChild(header);

        let subheader = DOMUtil.createElement({
            type: 'small',
            innerHTML: 'relativos al máximo histórico'
        });
        header.appendChild(subheader);

        this._relOverviewCard = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-card'
        });
        overview.appendChild(this._relOverviewCard);
    }

    // Create the history Element for the specified data and append it
    // to the history data container. If the history data
    // container doesnt exists, create one.
    //
    // @data - The data to create the history overview Element.
    setHistoryItem(data) {
        if(this._chart === undefined) {
            this._addHistory();
        }

        let card = this._chart.card;
        let header = card.querySelector('.mp-dropdown-sel');
        let mHeader = card.querySelector('.mp-dropdown-menu-header');
        let menu = card.querySelector('.mp-dropdown-menu');

        if(header.innerHTML === '') {
            header.innerHTML = data.name;
            mHeader.innerHTML = data.name;
            this.updateHistoryItem(data);
        }

        let menuItem = DOMUtil.createElement({
            type: 'div',
            class: 'mp-dropdown-item',
            innerHTML: data.name
        });
        menuItem.dataset.name = data.name;

        menuItem.addEventListener('click', e => {
            header.innerHTML = data.name;
            mHeader.innerHTML = data.name;

            EventDispatcher.dispatch(`data-${this._id}-selected`, this, data.name);
        });
        menu.appendChild(menuItem);
    }

    // Update the history Element.
    //
    // @data - The data to update the history Element.
    updateHistoryItem(data) {
        let cData = [];
        let formatTime = d3.timeParse('%e-%m-%Y');

        data.values.forEach(d => {
            let date = new Date(d.timestamp * 1000);
            let time = date.getDate() + '-' + date.getMonth() + '-' + date.getFullYear();

            cData.push({
                name: data.name,
                value: d.value,
                timestamp: d.timestamp,
                time: formatTime(time)
            });
        });

        this._chart.data = cData;
    }

    // Remove the history Element.
    //
    // @name - The data name to remove Element.
    removeHistoryItem(name) {
        let dropdowns = document.querySelectorAll('div.mp-dropdown-item');

        [].forEach.call(dropdowns, d => {
            if(d.innerHTML === name) {
                d.remove();
            }
        });
    }

    // Remove all data from the chart.
    clearHistoryItem() {
        this._chart.interval.stop();
        this._chart.data = [];

        this._clearChart();

        this._chart.interval = d3.interval(e => {
            this._updateChart();
        }, 1000);
    }

    // Create the history Element container, and initializes the chart.
    _addHistory() {
        let history = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-history'
        });
        this._element.appendChild(history);

        let header = DOMUtil.createElement({
            type: 'h4',
            class: 'mp-data-overview-header',
            innerHTML: 'Datos Históricos'
        });
        history.appendChild(header);

        let historyCard = DOMUtil.createElement({
            type: 'div',
            class: 'mp-data-card mp-history-card'
        });
        history.appendChild(historyCard);

        let dropdown = DOMUtil.createElement({
            type: 'div',
            class: 'mp-dropdown'
        });
        historyCard.appendChild(dropdown);

        let dropdownBtn = DOMUtil.createElement({
            type: 'div',
            class: 'mp-dropdown-btn'
        });
        dropdown.addEventListener('click', e => {
            dropdown.classList.toggle('show-dropdown');
        });
        dropdown.appendChild(dropdownBtn);

        let dropdownSelected = DOMUtil.createElement({
            type: 'span',
            class: 'mp-dropdown-sel'
        });
        dropdownBtn.appendChild(dropdownSelected);

        let dropdownArrow = DOMUtil.createElement({
            type: 'i',
            class: 'material-icons',
            innerHTML: 'arrow_drop_down'
        });
        dropdownBtn.appendChild(dropdownArrow);

        let dropdownMenu = DOMUtil.createElement({
            type: 'div',
            class: 'mp-dropdown-menu'
        });
        dropdown.appendChild(dropdownMenu);

        let dropdownMenuHeader = DOMUtil.createElement({
            type: 'div',
            class: 'mp-dropdown-menu-header'
        });
        dropdownMenu.appendChild(dropdownMenuHeader);

        let chart = DOMUtil.createElement({
            type: 'div',
            class: 'mp-chart-container'
        });
        historyCard.appendChild(chart);

        this._chart = {
            card: historyCard,
            chart: chart
        };

        this._addChart();

        this._chart.interval = d3.interval(e => {
            this._updateChart();
        }, 1000)
    }

    // Create the chart Element using d3js.
    _addChart() {
        let c = this._chart;

        c.margin = { top: 10, right: 5, bottom: 50, left: 30 };
        c.width = 300 - c.margin.left - c.margin.right;
        c.height = 200 - c.margin.top - c.margin.bottom;
        c.parseTime = d3.timeParse('%e-%m-%Y');
        c.barWidth = 5;

        c.x = d3.scaleTime().range([0, c.width]);

        c.y = d3.scaleLinear().range([c.height, 0]);

        c.xAxis = d3.axisBottom(c.x)
            .ticks(5)
            .tickSizeInner(0)
            .tickSizeOuter(0)
            .tickPadding(10);

        c.yAxis = d3.axisLeft(c.y)
            .ticks(5)
            .tickSizeInner(-c.width)
            .tickSizeOuter(0)
            .tickPadding(10);

        c.chart = d3.select(c.chart).append('svg')
            .attr('width', c.width + c.margin.left + c.margin.right)
            .attr('height', c.height + c.margin.top + c.margin.bottom)
            .append("g")
                .attr('transform', 'translate(' + c.margin.left + ',' + c.margin.top + ')');

        c.chart.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + c.height + ")")
                .call(c.xAxis);

        c.chart.append("g")
            .attr("class", "y axis")
                .call(c.yAxis);
    }

    // Update the chart, removing the data outside the X domain, updating the
    // old data inside the X domain, and appending the new data inside the
    // X domain.
    _updateChart() {
        const t = d3.transition().ease(d3.easeLinear).duration(1000);
        const c = this._chart;
        const data = c.data.slice(c.data.length - 10);

        const rect = c.chart.selectAll('rect')
            .data(data, d => { return d.time; });

        const updatedLength = rect.size();

        c.x.domain(d3.extent(data, function(d) { return d.time; }));
        c.y.domain([0, d3.max(data, d => { return d.value; })]);

        rect.exit()
            .remove();

        c.chart.selectAll('.x.axis')
            .transition(t)
                .call(c.xAxis);

        c.chart.selectAll('.y.axis')
            .transition(t)
                .call(c.yAxis);

        rect.transition(t)
            .attr('x', d => { return c.x(d.time); })
            .attr('y', d => { return c.y(d.value); })
            .attr('height', function(d) { return c.height - c.y(d.value); });

        rect.enter().append('rect')
                .attr('x', (d, i) => {
                    let offset;
                    if(updatedLength < 10) {
                        offset = c.x(d.time);
                    }
                    else {
                        offset = 2 * c.x(d.time) - c.x(data[i - 1].time);
                    }
                    return offset;
                })
                .attr('y', c.height)
                .attr('height', 0)
                .attr('width', 5)
                .style('fill', '#FF5252')
            .transition(t)
                .attr('x', d => { return c.x(d.time); })
                .attr('y', d => { return c.y(d.value); })
                .attr('height', function(d) { return c.height - c.y(d.value); });
    }

    // Remove al data from the chart and the axis.
    _clearChart() {
        const t = d3.transition().ease(d3.easeLinear).duration(500);
        const c = this._chart;

        const rect = c.chart.selectAll('rect')
            .data([], d => { return d; });

        rect.exit()
            .transition(t)
                .attr('y', c.height)
                .attr('height', 0)
            .remove();
    }

    get element() {
        return this._element;
    }
}

module.exports = MenuDataView;
