"use strict"

var map = require('../models/map.js');

var DOMUtil = require('../utils/mp-dom-util.js');
var EventDispatcher = require('../event-dispatcher.js');

// @class MenuItemView.
//
// Create the DOM Element tree that represents and renders the menu item of a
// node. This Element will be rendered inside the Main Menu container Element.
class MenuItemView {
    constructor(id) {
        this._id = id;
        this._clonedElements = [];

        this._element = DOMUtil.createElement({
            type: 'div',
            class: 'mp-menu-item'
        });

        this._addContent();

        this.elementSelected = this._elementSelected.bind(this);
        this.toggleVisibility = this._toggleVisibility.bind(this);

        this._setupEventListeners();
    }

    // Append this item menu view to the Main Menu View container submenu.
    //
    // @menu - The menu that will append this Menu Item
    addTo(menu) {
        menu.appendChild(this._element);
    }

    // Append the chidren of this node (because this node is a cluster) inside
    // this Element children container Element. To do so, its necessary to
    // perfom a deep clone of the child view, because the DOM tree cannot
    // render the same DOM Element twice.
    //
    // @views - The Menu Item Views to be cloned and appended to this one.
    appendChildren(views) {
        views.forEach(v => {
            let clone = v.cloneElement();
            this._childList.appendChild(clone.element);
        });
    }

    // Clone this view with all its properties and current state, and add to
    // it the necessary listeners.
    cloneElement() {
        let clone = new MenuItemView(this._id);

        this._clonedElements.push(clone);
        this._updateClones();

        return clone;
    }

    // Remove all the children from this Menu Item View, removing the cloned
    // views.
    //
    // @views - The Menu Item Views to be removed from this one.
    removeChildren(views) {
        views.forEach(v => {
            v.removeClone(this._childList);
        });
    }

    // Remove the clone element from the DOM tree.
    //
    // @parent - The DOM direct parent of the Element.
    removeClone(parent) {
        let menu = document.querySelector('#mp-menu');
        let view = this._clonedElements
            .find(e => e.element.parentNode == parent
                || e.element.parentNode == menu);

        if(view !== undefined) {
            view.remove();
        }
    }

    // Add a state to the menu. If state icon is true, create a SVGElement
    // and append it to the node Element. Otherwise change the color of the
    // menu icon Element.
    addState(state) {
        this._state = state;

        this._stateIcon = state.icon
            ? DOMUtil.createSVG({
                type: 'circle',
                height: 10,
                width: 10,
                color: state.color
            })
            : this._icon;

        this._stateName = DOMUtil.createElement({
            type: 'span',
            class: 'mp-menu-item-secondary',
            innerHTML: state.name
        });
        this._main.appendChild(this._stateName);

        if(state.icon) {
            this._content.appendChild(this._stateIcon);
            this._stateIcon = this._stateIcon.getElementsByTagName('circle')[0];
            this._stateIcon.style.fill = state.color;
        }
        else {
            this._stateIcon.style.color = state.color;
        }

        this._stateName.innerHTML = state.name;
    }

    // Remove all the Elements of this view clones included, and their
    // respective listeners.
    remove() {
        this._element.remove();

        this._element.removeEventListener('click', this.elementSelected);
        this._element.removeEventListener('click', this.toggleVisibility);

        this._clonedElements.forEach(c => {
            c.remove();
        });
    }

    // Create all the necessary Elements of a Menu Item View and append them
    // to the top level container.
    _addContent() {
        this._content = DOMUtil.createElement({
            type: 'div',
            class: 'mp-menu-item-content'
        });
        this._element.appendChild(this._content);

        this._icon = DOMUtil.createElement({
            type: 'i',
            class: 'material-icons mp-menu-icon'
        });
        this._content.appendChild(this._icon);

        this._main = DOMUtil.createElement({
            type: 'div',
            class: 'mp-menu-item-main'
        });
        this._content.appendChild(this._main);

        this._name = DOMUtil.createElement({
            type: 'span',
            class: 'mp-menu-item-primary'
        });
        this._main.appendChild(this._name);

        this._visibilityState = DOMUtil.createElement({
            type: 'i',
            class: 'material-icons mp-menu-visibility',
            innerHTML: 'visibility'
        });
        this._content.appendChild(this._visibilityState);

        this._childList = DOMUtil.createElement({
            type: 'div',
            class: 'mp-menu-item-children'
        });
        this._element.appendChild(this._childList);
    }

    // Fired when the user selects this menu item.
    _elementSelected() {
        DOMUtil.toggleClass(this._element, 'node-selected');
        EventDispatcher.dispatch(`node-${this._id}-selected`, this);
    }

    // Toggle the visibility state of this node, and prevents the click event
    // to propagate throught the parents of this Element.
    _toggleVisibility(e) {
        e.stopPropagation();
        EventDispatcher.dispatch(`node-${this._id}-toggle-visibility`, this);
    }

    // Whenever this Menu Item is updated, all the clones of this Menu Item are
    // updated too.
    _updateClones() {
        this._clonedElements.forEach(c => {
            c.name = this.name;
            c.icon = this.icon;
            if(this._state !== undefined) {
                c.addState(this._state);
            }
        });
    }

    _setupEventListeners() {
        this._element.addEventListener('click', this.elementSelected);
        this._visibilityState.addEventListener('click', this.toggleVisibility);
    }

    get element() {
        return this._element;
    }

    set name(name) {
        this._name.innerHTML = name;
        this._clonedElements.forEach(c => {
            c.name = name;
        });
    }

    get name() {
        return this._name.innerHTML;
    }

    set icon(icon) {
        this._icon.innerHTML = icon;
        this._clonedElements.forEach(c => {
            c.icon = icon;
        });
    }

    get icon() {
        return this._icon.innerHTML;
    }

    set state(state) {
        this._state = state;
        this._stateIcon.style.color = state.color;
        this._stateName.innerHTML = state.name;
        this._clonedElements.forEach(c => {
            c.state = state;
        });
    }
}

module.exports = MenuItemView;
